Immers.InternalConditions = {
	['ALWAYS'] 				= true,
	['COMBAT'] 				= false,
	['NONCOMBAT'] 			= false,
	['SNEAK'] 				= false,
	['MOUNT'] 				= false,
	['SPRINT'] 				= false,
	['MOUSE'] 				= false,
	['GAMEMENU'] 			= false,
	['INTERACT'] 			= false,
	['AIM_INTERACT'] 		= false,
	['AIM_NPC'] 			= false,
	['AIM_ENEMY'] 			= false,
	['AIM_PLAYER'] 			= false,
	['HURT'] 				= false,
	['ATTR'] 				= false,
	['DEBUFF'] 				= false,
	['DUNGMAP'] 			= false,
	['CITYMAP'] 			= false,
	['ZONEMAP'] 			= false,
	['ZONETRANSIT'] 		= false,
	['CITYENTER'] 			= false,
	['CITYEXIT'] 			= false,
}

Immers.SettingsConditions = {
	['Always'] = 						'ALWAYS',
	['In combat'] = 					'COMBAT',
	['Out of combat'] = 				'NONCOMBAT',
	['Sneaking'] = 						'SNEAK',
	['Mounted'] = 						'MOUNT',
	['Running'] = 						'SPRINT',
	['Mouse visible'] = 				'MOUSE',
	['GameMenu visible'] = 				'GAMEMENU',
	['Interacting (crafting, dialog)'] ='INTERACT',
	['Aiming at interactable'] = 		'AIM_INTERACT',
	['Aiming at npc'] = 				'AIM_NPC',
	['Aiming at enemy'] = 				'AIM_ENEMY',
	['Aiming at player'] = 				'AIM_PLAYER',
	['HP below full'] = 				'HURT',
	['Any attribute below full'] = 		'ATTR',
	['Debuffed'] = 						'DEBUFF',
	['In dungeon'] = 					'DUNGMAP',
	['In city'] = 						'CITYMAP',
	['Outside of city'] = 				'ZONEMAP',
	['Transiting between zones'] = 		'ZONETRANSIT',
	['Entering a city'] = 				'CITYENTER',
	['Exiting a city'] = 				'CITYEXIT',
}
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
local delayed_calls = {}

local function _DelayCall(name, delay_ms, func)
	local unique_name = Immers.name .. "." .. name

	if delayed_calls[unique_name] then EVENT_MANAGER:UnregisterForUpdate(unique_name) end
	delayed_calls[unique_name] = true

	EVENT_MANAGER:RegisterForUpdate(unique_name, delay_ms,
		function()
			EVENT_MANAGER:UnregisterForUpdate(unique_name)
			delayed_calls[unique_name] = nil
			func()
		end)
end

function Immers._SetupConditionalEvents()
	EVENT_MANAGER:RegisterForEvent(Immers.name, EVENT_EFFECT_CHANGED, Immers._EVENT_EFFECT_CHANGED)
	EVENT_MANAGER:RegisterForEvent(Immers.name, EVENT_POWER_UPDATE, Immers._EVENT_POWER_UPDATE)
	EVENT_MANAGER:RegisterForEvent(Immers.name, EVENT_STEALTH_STATE_CHANGED, Immers._EVENT_STEALTH_STATE_CHANGED)
	EVENT_MANAGER:RegisterForEvent(Immers.name, EVENT_RETICLE_TARGET_CHANGED, Immers._EVENT_RETICLE_TARGET_CHANGED)
end

Immers._EVENT_RETICLE_TARGET_CHANGED = function(_)
	local unitExists, unit_type, unit_reaction = DoesUnitExist('reticleover'), GetUnitType('reticleover'), GetUnitReaction('reticleover')
	
	local InternalStates = Immers.InternalConditions
	InternalStates.AIM_NPC = unitExists and unit_type == COMBAT_UNIT_TYPE_PLAYER_PET
	InternalStates.AIM_PLAYER = unitExists and unit_type == COMBAT_UNIT_TYPE_PLAYER
	InternalStates.AIM_ENEMY = unitExists and unit_reaction == UNIT_REACTION_HOSTILE
end

Immers._EVENT_STEALTH_STATE_CHANGED = function(_, unitTag, isStealthed)
	if unitTag ~= 'player' then return end

	local InternalStates = Immers.InternalConditions
	InternalStates.SNEAK = (isStealthed ~= 0)
end

local attr_tracker = {}

Immers._EVENT_POWER_UPDATE = function(_, unitTag, powerIndex, powerType, powerValue, powerMax, powerEffectiveMax)
	if unitTag ~= 'player' then return end

	attr_tracker[powerType] = (powerValue < powerMax)

	local InternalStates = Immers.InternalConditions
	InternalStates.HURT = attr_tracker[POWERTYPE_HEALTH]
	InternalStates.ATTR = attr_tracker[POWERTYPE_HEALTH] or attr_tracker[POWERTYPE_MAGICKA] or attr_tracker[POWERTYPE_STAMINA] or attr_tracker[POWERTYPE_MOUNT_STAMINA]
end

local buff_tracker = {}
local debuff_tracker = {}

Immers._EVENT_EFFECT_CHANGED = function(_, _, _, effectName, unitTag, effect_begin, effect_end, _, _, _, buffeffecttype)
	if unitTag ~= 'player' then return end

	if buffeffecttype == BUFF_EFFECT_TYPE_BUFF then
		if effect_begin ~= 0 and effect_end ~= 0 then
			buff_tracker[effectName] = true
		else
			buff_tracker[effectName] = nil
		end
	elseif buffeffecttype == BUFF_EFFECT_TYPE_DEBUFF then
		if effect_begin ~= 0 and effect_end ~= 0 then
			debuff_tracker[effectName] = true
		else
			debuff_tracker[effectName] = nil
		end
	end

	local InternalStates = Immers.InternalConditions
	local function isOneEntry(t)
		for k,v in pairs(t) do
			return true
		end
		return false
	end

	InternalStates.DEBUFF = isOneEntry(debuff_tracker)
	-- d("Debuff: ", tostring(isOneEntry(debuff_tracker)))
	-- d("Buff: ", tostring(isOneEntry(buff_tracker)))
end

local zone_data = {}

Immers.LocationCheck = function(_)
	if SCENE_MANAGER.currentScene.name == 'worldMap' then return end
	local InternalStates = Immers.InternalConditions

	local mapType, mapContentType = GetMapType(), GetMapContentType()
	InternalStates.ZONEMAP = mapType == MAPTYPE_ZONE
	InternalStates.CITYMAP = mapType == MAPTYPE_SUBZONE
	InternalStates.DUNGMAP = mapContentType == MAP_CONTENT_DUNGEON
	--d("Zone: " .. tostring(mapType == MAPTYPE_ZONE) .. "\nCity: " .. tostring(mapType == MAPTYPE_SUBZONE))

	if zone_data.pMapType ~= mapType then
		local EndCityTransit = function()
			if InternalStates.CITYEXIT or InternalStates.CITYENTER then
				InternalStates.CITYEXIT = false
				InternalStates.CITYENTER = false
			end
		end
		if InternalStates.ZONEMAP then
			InternalStates.CITYEXIT = true
		else
			InternalStates.CITYENTER = true
		end
		zone_data.pMapType = mapType
		_DelayCall("ImmersionToggles_CITY_TRANSIT_end", Immers.misc.transitTime, EndCityTransit)
	end

	local zoneIndex = GetUnitZoneIndex('player')
	if zoneIndex ~= zone_data.pZoneIndex then
		local EndZoneTransit = function()
			if InternalStates.ZONETRANSIT then
				InternalStates.ZONETRANSIT = false
			end
		end
		InternalStates.ZONETRANSIT = true
		zone_data.pZoneIndex = zoneIndex
		_DelayCall("ImmersionToggles_ZONE_TRANSIT_end", Immers.misc.transitTime, EndZoneTransit)
	end
end
----------------------------------------------------------------------------------------------------------------
Immers._UpdateInternalStates = function() -- split this up into a seperate file of events
	local cgTime = GetGameTimeMilliseconds()
	local InternalStates = Immers.InternalConditions
	-- AIM_INTERACT : aiming at interactable
	local interactionPossible, questInteraction, questTargetBased, questJournalIndex, questToolIndex, questToolOnCooldown = GetGameCameraInteractableInfo()
	InternalStates.AIM_INTERACT = questInteraction or questTargetBased
	-- InMouse, InGameMenu, InInteracting : is in menu, using mouse or in interaction interface
	local cScene = SCENE_MANAGER.currentScene.name
	InternalStates.MOUSE = (cScene == "hudui")
	InternalStates.GAMEMENU = (cScene ~= "hud") and (cScene ~= "hudui") and (cScene ~= "interact")
	InternalStates.INTERACT = (cScene == "interact")
	-- InCombat, InOutOfCombat : player in or out of combat
	local inCombat = IsUnitInCombat('player')
	InternalStates.NONCOMBAT = not inCombat
	InternalStates.COMBAT = inCombat
	-- InMount : player riding mount
	InternalStates.MOUNT = IsMounted()
	-- Sprint : player running
	InternalStates.SPRINT = IsShiftKeyDown() and IsPlayerMoving()
	-- Map/City Change/Transit (temporarily in hear until we can get a proper event or hook   
	Immers.LocationCheck()
	-- update visual status
	Immers._UpdateItems()
	-- return time taken to do all this
	return GetGameTimeMilliseconds() - cgTime
end