local ResetElement = function(element_name)
	Immers.CurSV.Toggles[element_name] = nil
end

local ExistsElement = function(element_name)
	return (Immers.CurSV.Toggles[element_name] ~= nil)
end

local Current = {
	Element = "",
	Condition = "",

	hide_EditBoxContents = "",
	show_EditBoxContents = "",
}

local ElementSelected = function()
	return Current.Element ~= ""
end

local ConditionSelected = function()
	return Current.Condition ~= ""
end

local CanSelectElement = function()
	if ElementSelected() and ExistsElement(Current.Element) then
		return false
	else
		return true
	end
end

local CanSetCondition = function()
	return not (not CanSelectElement() and ConditionSelected())
end

local RefreshElementState = function(element_name)
	if not Immers.Elements[element_name] then return end

	if Immers.CurSV.Toggles[Current.Element].disabled then
		Immers.Elements[element_name]:disableFunc()
	else
		Immers.Elements[element_name]:enableFunc()
	end
end

local function ReloadEditboxes()
	Current.show_EditBoxContents = ""
	Current.hide_EditBoxContents = ""

	if not CanSelectElement() then
		for k,v in pairs(Immers.SettingsConditions) do
			if not Immers.CurSV.Toggles[Current.Element].alwaysHide then Immers.CurSV.Toggles[Current.Element].alwaysHide = {} end
			if Immers.CurSV.Toggles[Current.Element].alwaysHide[v] then
				if Current.hide_EditBoxContents ~= '' then Current.hide_EditBoxContents = Current.hide_EditBoxContents .. ', ' end
				Current.hide_EditBoxContents = Current.hide_EditBoxContents .. '*' .. k
			end
			if Immers.CurSV.Toggles[Current.Element].hideWhen[v] then
				if Current.hide_EditBoxContents ~= '' then Current.hide_EditBoxContents = Current.hide_EditBoxContents .. ', ' end
				Current.hide_EditBoxContents = Current.hide_EditBoxContents .. k
			end
			if Immers.CurSV.Toggles[Current.Element].showWhen[v] then
				if Current.show_EditBoxContents ~= '' then Current.show_EditBoxContents = Current.show_EditBoxContents .. ', ' end
				Current.show_EditBoxContents = Current.show_EditBoxContents .. k
			end
		end
	end
end
----------------------------------------------------------------------------------------------------------------
----------------------------------------------------------------------------------------------------------------
function Immers._CreateSettingsMenu()
	local LAM2 = LibStub("LibAddonMenu-2.0")

	local panelData = {
		type = "panel",
		name = Immers.name,
		author = Immers.author,
		version = Immers.version,
		registerForRefresh = true,
	}
	local optionsData = {
		{
			type = "checkbox",
			name = "Use Account-Wide settings",
			getFunc = function()
				return Immers.AWSV.useAcctSettings
			end,
			setFunc = function(val)
				if val then
					Immers.AWSV.version = nil
					Immers.AWSV = ZO_SavedVars:NewAccountWide(Immers.settings_name,
						Immers.settings_version, nil, ImmersionToggles_Settings['Default'][GetDisplayName()][GetUnitName('player')])
					Immers.CurSV = Immers.AWSV
				else
					Immers.CSV.version = nil
					Immers.CSV = ZO_SavedVars:New(Immers.settings_name,
						Immers.settings_version, nil, ImmersionToggles_Settings['Default'][GetDisplayName()]['$AccountWide'])
					Immers.CurSV = Immers.CSV
				end
				Immers.AWSV.useAcctSettings = val
			end,
		},
		{
			type = 'description',
			title = 'Information',
			text = "|cFFFFFFTo manage an element: 1. Select the element you want to manage, 2. uncheck 'Don't manage this element', 3. then select a condition and add it as hide or show condition, repeat from 3." .. 
			'\n\n|cFF0000Conditions will be prioritized in the following order: [*hide -> show -> hide], e.g. setting the reticle to hide in combat and show in combat will cause it to always be shown in combat, unless *hide is used in which case it will hide.',
			width = 'full',
		},
		{ -----------------------move to top
			type = "slider",
			name = "How long Transitional conditions last (seconds):",
			width = "full",
			min = 1,
			max = 15,
			getFunc = function() return Immers.CurSV.transitTime end,
			setFunc = function(val)
				Immers.CurSV.transitTime = val
				Immers.misc.transitTime = (Immers.CurSV.transitTime * 1000)
			end,
			tooltip = "This is how long the Entering/Exiting and Transiting conditions will last.",
		},
		{
			type = "dropdown",
			name = "Select a Element:",
			choices = Terril_lib.keys(Immers.CurSV.Toggles),
			sort = "name-down",
			getFunc = function()
				return (Current.Element or "")
			end,
			setFunc = function(val)
				Current.Element = val
				ReloadEditboxes()
			end,
			width = "half",
		},
		{
			type = "checkbox",
			name = "Manage this element: ",
			getFunc = function()
				return ExistsElement(Current.Element) and (not Immers.CurSV.Toggles[Current.Element].disabled)
			end,
			setFunc = function(val)
				Immers.CurSV.Toggles[Current.Element].disabled = not val
				RefreshElementState(Current.Element)
			end,
			disabled = function() return Immers.Elements[Current.Element] == nil end,
			width = 'half',
		},
		{
			type = "dropdown",
			name = "Select a Condition:",
			sort = "name-down",
			choices = Terril_lib.keys(Immers.SettingsConditions),
			getFunc = function()
				return Current.Condition
			end,
			setFunc = function(val)
				Current.Condition = val
			end,
			width = 'full',
			disabled = CanSelectElement,
		},
		{
			type = "editbox",
			name = 'Will hide when',
			width = "full",
			isExtraWide = true,
			isMultiline = false,
			getFunc = function() return Current.hide_EditBoxContents end,
			setFunc = function(val)
				return
			end,
			disabled = CanSelectElement,
			tooltip = "This editbox is only for display, editing it does nothing.",
		},
		{
			type = "button",
			name = "Add as hide",
			tooltip = "Adds the currently selected hide condition",
			func = function()
				local internal_state = Immers.SettingsConditions[Current.Condition]
				Immers.CurSV.Toggles[Current.Element].alwaysHide[internal_state] = nil
				Immers.CurSV.Toggles[Current.Element].hideWhen[internal_state] = true
				ReloadEditboxes()
			end,
			width = "half",
			disabled = CanSetCondition,
		},
		{
			type = "button",
			name = "Remove condition",
			tooltip = "Removes the currently selected hide condition",
			func = function()
				local internal_state = Immers.SettingsConditions[Current.Condition]
				Immers.CurSV.Toggles[Current.Element].hideWhen[internal_state] = nil
				Immers.CurSV.Toggles[Current.Element].alwaysHide[internal_state] = nil
				ReloadEditboxes()
			end,
			width = "half",
			disabled = CanSetCondition,
		},
		{
			type = "button",
			name = "Add as *hide",
			tooltip = "Adds the currently selected hide condition, and will take priority over show conditions.",
			func = function()
				local internal_state = Immers.SettingsConditions[Current.Condition]
				Immers.CurSV.Toggles[Current.Element].alwaysHide[internal_state] = true
				Immers.CurSV.Toggles[Current.Element].hideWhen[internal_state] = nil
				ReloadEditboxes()
			end,
			width = "half",
			disabled = CanSetCondition,
		},
		{
			type = "button",
			name = "Remove All",
			tooltip = "Removes all hide conditions on this element",
			func = function()
				Immers.CurSV.Toggles[Current.Element].hideWhen = {}
				Immers.CurSV.Toggles[Current.Element].alwaysHide = {}
				ReloadEditboxes()
			end,
			width = "half",
			disabled = CanSelectElement,
		},
		{
			type = "editbox",
			name = 'Will show when',
			width = "full",
			isExtraWide = true,
			isMultiline = false,
			getFunc = function() return Current.show_EditBoxContents end,
			setFunc = function(val)
				return
			end,
			disabled = CanSelectElement,
			tooltip = "This editbox is only for display, editing it does nothing.",
		},
		{
			type = "button",
			name = "Add as show",
			tooltip = "Adds the currently selected show condition",
			func = function()
				local internal_state = Immers.SettingsConditions[Current.Condition]
				Immers.CurSV.Toggles[Current.Element].showWhen[internal_state] = true
				ReloadEditboxes()
			end,
			width = "half",
			disabled = CanSetCondition,
		},
		{
			type = "button",
			name = "Remove condition",
			tooltip = "Removes the currently selected show condition",
			func = function()
				local internal_state = Immers.SettingsConditions[Current.Condition]
				Immers.CurSV.Toggles[Current.Element].showWhen[internal_state] = nil
				ReloadEditboxes()
			end,
			width = "half",
			disabled = CanSetCondition,
		},
		{
			type = 'custom',
			width = 'half',
		},
		{
			type = "button",
			name = "Remove All",
			tooltip = "Removes all show conditions on this element",
			func = function()
				Immers.CurSV.Toggles[Current.Element].showWhen = {}
				ReloadEditboxes()
			end,
			width = "half",
			disabled = CanSelectElement,
		},
		{
			type = 'header',
			width = 'full',
		},
		{
			type = "button",
			name = "Reset this element",
			tooltip = "Resets the settings for this element; If the element is no no longer installed, it will remove the settings.",
			func = function()
				ResetElement(Current.Element)
				ReloadUI()
			end,
			width = 'half',
			disabled = CanSelectElement,
		},
		{
			type = "button",
			name = "Reset all settings",
			tooltip = "Resets all element settings and conditions.",
			func = function()
				for i,v in ipairs(Terril_lib.keys(Immers.CurSV.Toggles)) do
					ResetElement(v)
				end
				ReloadUI()
			end,
			width = "half",
		},
	}

	LAM2:RegisterAddonPanel(Immers.name .. "OptionsPanel", panelData)
	LAM2:RegisterOptionControls(Immers.name .. "OptionsPanel", optionsData)
end

